#!/usr/bin/env python

from moinformat import make_linker, make_output, make_parser, make_serialiser, \
                       parse, serialise
from os.path import split
import sys

def main():
    dirname, progname = split(sys.argv[0])
    args = sys.argv[1:]

    l = filenames = []
    formats = []
    pagenames = []
    mappings = []
    outputs = []
    tree = False
    macros = False

    for arg in args:

        # Detect tree output.

        if arg == "--tree":
            tree = True

        # Detect macro evaluation.

        elif arg == "--macros":
            macros = True

        # Switch to collecting formats.

        elif arg == "--format":
            l = formats
            continue

        # Switch to collecting mappings.

        elif arg == "--mapping":
            l = mappings
            continue

        # Switch to collecting output locations.

        elif arg == "--output":
            l = outputs
            continue

        # Switch to collecting page names.

        elif arg == "--pagename":
            l = pagenames
            continue

        # Collect options and arguments.

        else:
            l.append(arg)

            # Collect multiple mappings.

            if l is mappings:
                continue

        # Collect filenames normally.

        l = filenames

    format = formats and formats[0] or "html"

    # Derive the page name from the filename if not specified.

    filename = filenames[0]
    pagename = pagenames and pagenames[0] or split(filename)[-1]

    # Obtain an output context from any specified output details.

    output = outputs and make_output(outputs[0]) or None

    # Derive a proper mapping from the given list of values.

    mapping = {}
    key = None

    for arg in mappings:
        if key is None:
            key = arg
        else:
            mapping[key] = arg
            key = None

    # Open the file, parse the content, serialise the document.

    f = open(filename)
    try:
        p = make_parser()
        d = parse(f.read(), p)

        if macros:
            p.evaluate_macros()

        if tree:
            print d.prettyprint()
        else:
            l = make_linker(format, pagename, mapping)
            s = make_serialiser(format, output, l)
            print serialise(d, s)
    finally:
        f.close()

if __name__ == "__main__":
    main()

# vim: tabstop=4 expandtab shiftwidth=4
